/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.CommandRegistration;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.command.Exit;
import org.ow2.petals.cli.command.Help;
import org.ow2.petals.cli.command.Load;
import org.ow2.petals.cli.command.Print;
import org.ow2.petals.cli.command.Pwd;
import org.ow2.petals.cli.command.Set;
import org.ow2.petals.cli.command.SystemCall;
import org.ow2.petals.cli.shell.AbstractShell;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.cli.shell.command.Deploy;
import org.ow2.petals.cli.shell.command.Disconnect;
import org.ow2.petals.cli.shell.command.ListArtifacts;
import org.ow2.petals.cli.shell.command.ListRegistry;
import org.ow2.petals.cli.shell.command.LoggerSet;
import org.ow2.petals.cli.shell.command.Loggers;
import org.ow2.petals.cli.shell.command.MoveContainer;
import org.ow2.petals.cli.shell.command.PropertiesList;
import org.ow2.petals.cli.shell.command.ReloadConfiguration;
import org.ow2.petals.cli.shell.command.Show;
import org.ow2.petals.cli.shell.command.StartArtifact;
import org.ow2.petals.cli.shell.command.Stop;
import org.ow2.petals.cli.shell.command.StopArtifact;
import org.ow2.petals.cli.shell.command.TopologyList;
import org.ow2.petals.cli.shell.command.Undeploy;
import org.ow2.petals.cli.shell.command.Version;

/**
 * A unit test to check that Petals ESB CLI embeds all its commands, and only the expected commands.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
// TODO: The unit test core must be mutualized with the equivalent test of Petals ESB CLI
public class EmbeddedCommandsTest {

    private class MyShell extends AbstractShell {

        public MyShell() {
            super(null, null, false, false, null, null, "");
        }

        @Override
        public void run() {
            // NOP
        }

    }

    private final static Class<?>[] EXPECTED_COMMANDS = new Class[] { Connect.class, Deploy.class, Disconnect.class,
            Exit.class, Help.class, ListArtifacts.class, ListRegistry.class, Load.class, LoggerSet.class,
            Loggers.class, MoveContainer.class, Print.class, PropertiesList.class, Pwd.class,
            ReloadConfiguration.class, Set.class, Show.class, StartArtifact.class, Stop.class, StopArtifact.class,
            SystemCall.class, TopologyList.class, Undeploy.class, Version.class };

    @Test
    public void expectedCommands() {

        final Shell myShell = new MyShell();
        CommandRegistration.registers(myShell);

        final List<Class<?>> expectedCommands = new ArrayList<Class<?>>(Arrays.asList(EXPECTED_COMMANDS));
        final Map<String, Command> commands = myShell.getCommands();
        final Iterator<Command> itCommands = commands.values().iterator();
        while (itCommands.hasNext()) {
            final Command command = itCommands.next();

            final Iterator<Class<?>> itExpectedCommands = expectedCommands.iterator();
            while (itExpectedCommands.hasNext()) {
                final Class<?> expectedCommand = itExpectedCommands.next();
                if (expectedCommand.isInstance(command)) {
                    itExpectedCommands.remove();
                    itCommands.remove();
                    break;
                }
            }
        }

        assertEquals("Commands are not expected", 0, commands.size());
        assertEquals("Commands are missing", 0, expectedCommands.size());

        assertTrue(myShell.getConnectionCommand() instanceof Connect);
    }

}
