/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.completer;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.List;

import jline.console.completer.Completer;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.admin.ContainerAdministrationMock;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.junit.PetalsAdministrationApi;

/**
 * Unit tests of {@link ContainerNameCompleter}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ContainerNameCompleterTest {

    @Rule
    public final PetalsAdministrationApi adminApi = new PetalsAdministrationApi();

    @Test
    public void testComplete() throws DuplicatedServiceException, MissingServiceException {

        final Completer completer = new ContainerNameCompleter(PetalsAdministrationFactory.newInstance());

        // Check the completer when buffer is empty
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        completer.complete("", 0, candidates);
        assertNotNull(candidates);
        assertEquals("Unexpected candidate number", 2, candidates.size());
        boolean container0Found = false;
        boolean container1Found = false;
        for (final CharSequence candidate : candidates) {
            if (ContainerAdministrationMock.CONTAINER_00_NAME.equals(candidate)) {
                container0Found = true;
            } else if (ContainerAdministrationMock.CONTAINER_01_NAME.equals(candidate)) {
                container1Found = true;
            } else {
                fail("Unexpected candidate");
            }
        }
        assertTrue("Container 0 is missing", container0Found);
        assertTrue("Container 1 is missing", container1Found);

        // Check the completer when the buffer matches the beginning of an existing container name
        candidates.clear();
        completer.complete(ContainerAdministrationMock.CONTAINER_00_NAME.substring(0,
                ContainerAdministrationMock.CONTAINER_00_NAME.length() - 1), 0, candidates);
        assertNotNull(candidates);
        assertEquals("Unexpected candidate number", 2, candidates.size());
        container0Found = false;
        container1Found = false;
        for (final CharSequence candidate : candidates) {
            if (ContainerAdministrationMock.CONTAINER_00_NAME.equals(candidate)) {
                container0Found = true;
            } else if (ContainerAdministrationMock.CONTAINER_01_NAME.equals(candidate)) {
                container1Found = true;
            } else {
                fail("Unexpected candidate");
            }
        }
        assertTrue("Container 0 is missing", container0Found);
        assertTrue("Container 1 is missing", container1Found);

        // Check the completer when the buffer matches exactly an existing container name
        candidates.clear();
        completer.complete(ContainerAdministrationMock.CONTAINER_00_NAME, 0, candidates);
        assertNotNull(candidates);
        assertEquals("Unexpected candidate number", 1, candidates.size());
        // Note: A space is added when only one candidate is returned
        assertEquals("Unexpected candidate", ContainerAdministrationMock.CONTAINER_00_NAME + " ", candidates.get(0));

        // Check the completer when the buffer does not match the beginning of an existing container name
        candidates.clear();
        completer.complete("unknown", 0, candidates);
        assertNotNull(candidates);
        assertEquals("Unexpected candidate number", 0, candidates.size());
    }

}
